"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Helper class for modifying TypeDoc page content.
 */
var PageProcessor = /** @class */ (function () {
    /**
     * Creates a new PageProcessor with the content of the page.
     * @param content The content of the page.
     */
    function PageProcessor(content) {
        this._content = content;
    }
    Object.defineProperty(PageProcessor.prototype, "content", {
        /**
         * Returns the current content of the page.
         * @returns The current content of the page.
         */
        get: function () {
            return this._content;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PageProcessor.prototype, "isDetailPage", {
        /**
         * Checks if the page is detail page for a class or reflection.
         * @returns True, if the page is a detail page, otherwise false.
         */
        get: function () {
            return this.hasSection("Hierarchy" /* Hierarchy */);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Inserts new content above a given section.
     * @param section The section above which the content should be inserted.
     * @param newContent The content to insert.
     */
    PageProcessor.prototype.insertAboveSection = function (section, newContent) {
        var sectionLocation = this.getSectionLocation(section);
        if (sectionLocation.startIndex !== sectionLocation.endIndex) {
            this._content =
                this._content.substring(0, sectionLocation.startIndex) +
                    newContent +
                    this._content.substring(sectionLocation.startIndex);
        }
    };
    /**
     * Inserts new content below a given section.
     * @param section The section below which the content should be inserted.
     * @param newContent The content to insert.
     */
    PageProcessor.prototype.insertBelowSection = function (section, newContent) {
        var sectionLocation = this.getSectionLocation(section);
        if (sectionLocation.startIndex !== sectionLocation.endIndex) {
            this._content =
                this._content.substring(0, sectionLocation.endIndex) +
                    newContent +
                    this._content.substring(sectionLocation.endIndex);
        }
    };
    /**
     * Checks if the page has a section with the given name.
     * @param sectionName The name of the section.
     * @returns True, if the page has a section with the given name, otherwise false.
     */
    PageProcessor.prototype.hasSection = function (sectionName) {
        var regex = this.createRegexpForSection(sectionName);
        return regex.test(this._content);
    };
    /**
     * Returns a regular expression that can be used to match a section with a given name.
     * @param sectionName The name of the section.
     * @returns The regular expression to match the section.
     */
    PageProcessor.prototype.createRegexpForSection = function (sectionName) {
        return new RegExp('<section class="tsd-panel[^"<>]*?">\\s*<h3>' + sectionName + "</h3>[^]*?</section>");
    };
    /**
     * Returns the position of the section within the content.
     * @param sectionName The name of the section.
     * @returns An object with the start and end index of the section.
     *          If the section cannot be found in the content both startIndex and endIndex equal 0.
     */
    PageProcessor.prototype.getSectionLocation = function (sectionName) {
        var regex = this.createRegexpForSection(sectionName);
        var match = regex.exec(this._content);
        if (match) {
            return { startIndex: match.index, endIndex: match.index + match[0].length };
        }
        else {
            return { startIndex: 0, endIndex: 0 };
        }
    };
    return PageProcessor;
}());
exports.PageProcessor = PageProcessor;
//# sourceMappingURL=page_processor.js.map