"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var typedoc_1 = require("typedoc");
var converter_1 = require("typedoc/dist/lib/converter");
var events_1 = require("typedoc/dist/lib/output/events");
var image_generator_1 = require("./image_generator");
var page_processor_1 = require("./page_processor");
var plantuml_utils_1 = require("./plantuml_utils");
var plugin_options_1 = require("./plugin_options");
var typedoc_utils_1 = require("./typedoc_utils");
/**
 * The UML class diagram generator plugin.
 *
 * # What does it do?
 *
 * This plugin generates an UML class diagram for classes and interfaces.
 *
 * # How does it do it?
 *
 * 1. The plugin scans through the classes and interfaces and generates PlantUML code for them.
 * 2. The plugin uses the node-plantuml module to generate image files from the PlantUML code.
 * 3. The plugin adds a section with the image to the page of each class and interface.
 */
var Plugin = /** @class */ (function () {
    function Plugin() {
        /** The options of this plugin. */
        this.options = new plugin_options_1.PluginOptions();
        /** Used when the class diagrams are created locally. */
        this.localImageGenerator = new image_generator_1.ImageGenerator();
    }
    /**
     * Initializes the plugin.
     * @param typedoc The TypeDoc application.
     */
    Plugin.prototype.initialize = function (typedoc) {
        this.addOptionsToApplication(typedoc);
        this.subscribeToApplicationEvents(typedoc);
    };
    /**
     * Adds the plugin's options to the application's options.
     * @param typedoc The TypeDoc application.
     */
    Plugin.prototype.addOptionsToApplication = function (typedoc) {
        this.options.addToApplication(typedoc);
    };
    /**
     * Subscribes to events of the application so that the plugin can do its work
     * in the particular doc generation phases.
     * @param typedoc The TypeDoc application.
     */
    Plugin.prototype.subscribeToApplicationEvents = function (typedoc) {
        var _this = this;
        typedoc.converter.on(converter_1.Converter.EVENT_RESOLVE_BEGIN, function (c) { return _this.onConverterResolveBegin(c); });
        typedoc.renderer.on(events_1.RendererEvent.BEGIN, function (e) { return _this.onRendererBegin(e); });
        typedoc.renderer.on(events_1.PageEvent.END, function (e) { return _this.onRendererEndPage(e); });
        typedoc.renderer.on(events_1.RendererEvent.END, function (e) { return _this.onRendererEnd(e); });
    };
    /**
     * Triggered when the TypeDoc converter begins resolving a project.
     * Reads plugin parameter values.
     * @param context Describes the current state the converter is in.
     */
    Plugin.prototype.onConverterResolveBegin = function (context) {
        this.options.readValuesFromApplication(context.converter.owner.application);
    };
    /**
     * Triggered before the renderer starts rendering a project.
     * Setup image generator output directory and progress bar.
     * @param event The event emitted by the renderer class.
     */
    Plugin.prototype.onRendererBegin = function (event) {
        this.localImageGenerator.setOutputDirectory(path.join(event.outputDirectory, "assets/images/"));
    };
    /**
     * Triggered after a document has been rendered, just before it is written to disc.
     * Generates a UML class diagram and adds it to the page of the reflection.
     * @param event The event emitted by the renderer class.
     */
    Plugin.prototype.onRendererEndPage = function (event) {
        if (!event.contents) {
            return;
        }
        var page = new page_processor_1.PageProcessor(event.contents);
        if (!page.isDetailPage) {
            return;
        }
        var reflection = event.model;
        var plantUmlLines = this.shouldCreateClassDiagramForReflection(reflection)
            ? this.createClassDiagramPlantUmlForReflection(reflection)
            : [];
        if (plantUmlLines.length === 0) {
            return;
        }
        var encodedPlantUml = plantuml_utils_1.PlantUmlUtils.encode(plantUmlLines.join("\n"));
        var imagePath = this.options.outputImageLocation === plugin_options_1.ImageLocation.Local
            ? this.localImageGenerator.writeImage(event.filename, encodedPlantUml, this.options.outputImageFormat)
            : plantuml_utils_1.PlantUmlUtils.createPlantUmlServerUrl(encodedPlantUml, this.options.outputImageFormat);
        var hierarchyDiagramSection = this.createHierarchyDiagramSection(imagePath, reflection.name);
        if (this.options.umlClassDiagramPosition === plugin_options_1.ClassDiagramPosition.Above) {
            page.insertAboveSection("Hierarchy" /* Hierarchy */, hierarchyDiagramSection);
        }
        else {
            page.insertBelowSection("Hierarchy" /* Hierarchy */, hierarchyDiagramSection);
        }
        event.contents = page.content;
    };
    /**
     * Returns if a class diagram should be generated for the given reflection.
     * @param reflection The reflection for which the question is asked.
     * @returns True, if a class diagram should be generated for the given reflection, otherwise false.
     */
    Plugin.prototype.shouldCreateClassDiagramForReflection = function (reflection) {
        if ((this.options.umlClassDiagramType === plugin_options_1.ClassDiagramType.Simple ||
            this.options.umlClassDiagramType === plugin_options_1.ClassDiagramType.Detailed) &&
            reflection instanceof typedoc_1.DeclarationReflection &&
            (reflection.kind === typedoc_1.ReflectionKind.Class || reflection.kind === typedoc_1.ReflectionKind.Interface) &&
            typedoc_utils_1.TypeDocUtils.reflectionIsPartOfClassHierarchy(reflection)) {
            return true;
        }
        return false;
    };
    /**
     * Generates the Plant UML lines for the class diagram of the given reflection.
     * @param reflection The reflection for which to generate a class diagram.
     * @returns The Plant UML lines for the class diagram of the given reflection.
     *          If the given reflection is not part of an inheritance or implementation, the result is an empty array.
     */
    Plugin.prototype.createClassDiagramPlantUmlForReflection = function (reflection) {
        var includeChildren = this.options.umlClassDiagramType === plugin_options_1.ClassDiagramType.Detailed;
        var plantUmlLines = new Array();
        var siblingsAbove = 0;
        var siblingsBelow = 0;
        // add class/interface
        plantUmlLines = plantUmlLines.concat(plantuml_utils_1.PlantUmlUtils.createPlantUmlForReflection(reflection, includeChildren));
        // add classes/interfaces this type is extending
        var extendedTypes = typedoc_utils_1.TypeDocUtils.getExtendedTypesForReflection(reflection);
        for (var _i = 0, extendedTypes_1 = extendedTypes; _i < extendedTypes_1.length; _i++) {
            var type = extendedTypes_1[_i];
            plantUmlLines = plantUmlLines.concat(plantuml_utils_1.PlantUmlUtils.createPlantUmlForReflection(type, includeChildren));
            plantUmlLines.push(type.name + " <|-- " + reflection.name);
            ++siblingsAbove;
        }
        // add classes/interfaces this type is implementing
        var implementedTypes = typedoc_utils_1.TypeDocUtils.getImplementedTypesForReflection(reflection);
        for (var _a = 0, implementedTypes_1 = implementedTypes; _a < implementedTypes_1.length; _a++) {
            var type = implementedTypes_1[_a];
            plantUmlLines = plantUmlLines.concat(plantuml_utils_1.PlantUmlUtils.createPlantUmlForReflection(type, includeChildren));
            plantUmlLines.push(type.name + " <|.. " + reflection.name);
            ++siblingsAbove;
        }
        // add classes/interfaces that are extending this type
        var extendedBys = typedoc_utils_1.TypeDocUtils.getExtendedBysForReflection(reflection);
        for (var _b = 0, extendedBys_1 = extendedBys; _b < extendedBys_1.length; _b++) {
            var type = extendedBys_1[_b];
            plantUmlLines = plantUmlLines.concat(plantuml_utils_1.PlantUmlUtils.createPlantUmlForReflection(type, includeChildren));
            plantUmlLines.push(reflection.name + " <|-- " + type.name);
            ++siblingsBelow;
        }
        // add classes that are implementing this type
        var implementedBys = typedoc_utils_1.TypeDocUtils.getImplementedBysForReflection(reflection);
        for (var _c = 0, implementedBys_1 = implementedBys; _c < implementedBys_1.length; _c++) {
            var type = implementedBys_1[_c];
            plantUmlLines = plantUmlLines.concat(plantuml_utils_1.PlantUmlUtils.createPlantUmlForReflection(type, includeChildren));
            plantUmlLines.push(reflection.name + " <|.. " + type.name);
            ++siblingsBelow;
        }
        // Return no UML if there is no inheritance or implementation involved
        if (siblingsAbove + siblingsBelow === 0) {
            plantUmlLines = [];
        }
        else {
            if (this.options.umlClassDiagramHideEmptyMembers) {
                plantUmlLines.unshift("hide empty fields");
                plantUmlLines.unshift("hide empty methods");
            }
            if (this.options.umlClassDiagramHideCircledChar) {
                plantUmlLines.unshift("hide circle");
            }
            if (siblingsAbove > this.options.umlClassDiagramTopDownLayoutMaxSiblings ||
                siblingsBelow > this.options.umlClassDiagramTopDownLayoutMaxSiblings) {
                plantUmlLines.unshift("left to right direction");
            }
            if (this.options.umlClassDiagramMemberVisibilityStyle === plugin_options_1.ClassDiagramMemberVisibilityStyle.Text) {
                plantUmlLines.unshift("skinparam ClassAttributeIconSize 0");
            }
            if (this.options.umlClassDiagramHideShadow) {
                plantUmlLines.unshift("skinparam Shadowing false");
            }
            if (this.options.umlClassDiagramBoxBorderRadius) {
                plantUmlLines.unshift("skinparam RoundCorner " + this.options.umlClassDiagramBoxBorderRadius);
            }
            if (this.options.umlClassDiagramBoxBackgroundColor) {
                plantUmlLines.unshift("skinparam ClassBackgroundColor " + this.options.umlClassDiagramBoxBackgroundColor);
            }
            if (this.options.umlClassDiagramBoxBorderColor) {
                plantUmlLines.unshift("skinparam ClassBorderColor " + this.options.umlClassDiagramBoxBorderColor);
            }
            if (this.options.umlClassDiagramBoxBorderWidth >= 0) {
                plantUmlLines.unshift("skinparam ClassBorderThickness " + this.options.umlClassDiagramBoxBorderWidth);
            }
            if (this.options.umlClassDiagramArrowColor) {
                plantUmlLines.unshift("skinparam ClassArrowColor " + this.options.umlClassDiagramArrowColor);
            }
            if (this.options.umlClassDiagramClassFontName) {
                plantUmlLines.unshift("skinparam ClassFontName " + this.options.umlClassDiagramClassFontName);
            }
            if (this.options.umlClassDiagramClassFontSize) {
                plantUmlLines.unshift("skinparam ClassFontSize " + this.options.umlClassDiagramClassFontSize);
            }
            if (this.options.umlClassDiagramClassFontStyle !== plugin_options_1.FontStyle.Undefined) {
                plantUmlLines.unshift("skinparam ClassFontStyle " + this.options.umlClassDiagramClassFontStyle.toString());
            }
            if (this.options.umlClassDiagramClassFontColor) {
                plantUmlLines.unshift("skinparam ClassFontColor " + this.options.umlClassDiagramClassFontColor);
            }
            if (this.options.umlClassDiagramClassAttributeFontName) {
                plantUmlLines.unshift("skinparam ClassAttributeFontName " + this.options.umlClassDiagramClassAttributeFontName);
            }
            if (this.options.umlClassDiagramClassAttributeFontSize) {
                plantUmlLines.unshift("skinparam ClassAttributeFontSize " + this.options.umlClassDiagramClassAttributeFontSize);
            }
            if (this.options.umlClassDiagramClassAttributeFontStyle !== plugin_options_1.FontStyle.Undefined) {
                plantUmlLines.unshift("skinparam ClassAttributeFontStyle " +
                    this.options.umlClassDiagramClassAttributeFontStyle.toString());
            }
            if (this.options.umlClassDiagramClassAttributeFontColor) {
                plantUmlLines.unshift("skinparam ClassAttributeFontColor " + this.options.umlClassDiagramClassAttributeFontColor);
            }
        }
        return plantUmlLines;
    };
    /**
     * Creates HTML for a section containing a hierarchy diagram.
     * @param imagePath The hierarchy diagram as an image.
     * @param reflectionName The name of the reflection for which the hierarchy diagram was generated.
     * @returns The HTML for the section.
     */
    Plugin.prototype.createHierarchyDiagramSection = function (imagePath, reflectionName) {
        return "<section class=\"tsd-panel tsd-hierarchy\">\n                    <h3>" + this.options.umlClassDiagramSectionTitle + "</h3>\n                        <a class=\"uml-class\" href=\"" + imagePath + "\" title=\"Click to enlarge\">\n                            <img src=\"" + imagePath + "\"\n                                 alt=\"UML class diagram of " + reflectionName + "\" />\n                        </a>\n                </section>";
    };
    /**
     * Triggered after the renderer has written all documents.
     * Appends style data to the main CSS file.
     * @param event The event emitted by the renderer class.
     */
    Plugin.prototype.onRendererEnd = function (event) {
        var filename = path.join(event.outputDirectory, "assets/css/main.css");
        var data = fs.readFileSync(filename, "utf8") +
            "\n.uml-class { max-width:100%; display:block; margin:0 auto; text-align:center }\n";
        fs.writeFileSync(filename, data, "utf8");
    };
    return Plugin;
}());
exports.Plugin = Plugin;
//# sourceMappingURL=plugin.js.map